/*
 * Copyright (c) 2016-2022 VMware, Inc. All Rights Reserved.
 * This software is released under MIT license.
 * The full license information can be found in LICENSE in the root directory of this project.
 */

import '@cds/core/input/register.js';
import { html } from 'lit';
import { getElementStorybookArgs, spreadProps } from '@cds/core/internal';

export default {
  title: 'Stories/Input',
  component: 'cds-input',
  parameters: {
    options: { showPanel: true },
    design: {
      type: 'figma',
      url: 'https://www.figma.com/file/v2mkhzKQdhECXOx8BElgdA/Clarity-UI-Library---light-2.2.0?node-id=0%3A853',
    },
  },
};

export function API(args: any) {
  return html`
    <cds-input ...="${spreadProps(getElementStorybookArgs(args))}">
      <label>input</label>
      <input placeholder="placeholder text" />
      <cds-control-message .status=${args.status}>message text</cds-control-message>
    </cds-input>
  `;
}

/** @website */
export function input() {
  return html`
    <cds-input control-width="shrink">
      <label>input</label>
      <input placeholder="placeholder text" required />
    </cds-input>
  `;
}

/** @website */
export function inputWithLabelTextTranformation() {
  return html`
    <style>
      cds-input#cds-input-with-label-text-transfomration label {
        --text-transform: lowercase;
      }
    </style>
    <cds-input id="cds-input-with-label-text-transfomration" control-width="shrink">
      <label>input</label>
      <input placeholder="placeholder text" required />
    </cds-input>
  `;
}

/** @website */
export function vertical() {
  return html`
    <cds-form-group layout="vertical">
      <cds-input layout="vertical">
        <label>label</label>
        <input placeholder="name" />
        <cds-control-message>message text</cds-control-message>
      </cds-input>

      <cds-input layout="vertical">
        <label>disabled</label>
        <input placeholder="name" disabled />
        <cds-control-message>disabled message</cds-control-message>
      </cds-input>

      <cds-input layout="vertical">
        <label>error</label>
        <input placeholder="name" />
        <cds-control-message status="error">error message</cds-control-message>
      </cds-input>

      <cds-input layout="vertical">
        <label>success</label>
        <input placeholder="name" />
        <cds-control-message status="success">success message</cds-control-message>
      </cds-input>
    </cds-form-group>
  `;
}

/** @website */
export function horizontal() {
  return html`
    <cds-form-group layout="horizontal">
      <cds-input layout="horizontal">
        <label>label</label>
        <input placeholder="name" />
        <cds-control-message>message text</cds-control-message>
      </cds-input>

      <cds-input layout="horizontal">
        <label>disabled</label>
        <input placeholder="name" disabled />
        <cds-control-message>disabled message</cds-control-message>
      </cds-input>

      <cds-input layout="horizontal" status="error">
        <label>error</label>
        <input placeholder="name" />
        <cds-control-message status="error">error message</cds-control-message>
      </cds-input>

      <cds-input layout="horizontal" status="success">
        <label>success</label>
        <input placeholder="name" />
        <cds-control-message status="success">success message</cds-control-message>
      </cds-input>
    </cds-form-group>
  `;
}

/** @website */
export function compact() {
  return html`
    <cds-form-group layout="compact">
      <cds-input layout="compact">
        <label>label</label>
        <input placeholder="name" />
        <cds-control-message>message text</cds-control-message>
      </cds-input>

      <cds-input layout="compact">
        <label>disabled</label>
        <input placeholder="name" disabled />
        <cds-control-message>disabled message</cds-control-message>
      </cds-input>

      <cds-input layout="compact" status="error">
        <label>error</label>
        <input placeholder="name" />
        <cds-control-message status="error">error message</cds-control-message>
      </cds-input>

      <cds-input layout="compact" status="success">
        <label>success</label>
        <input placeholder="name" />
        <cds-control-message status="success">success message</cds-control-message>
      </cds-input>
    </cds-form-group>
  `;
}

export function readonly() {
  return html`
    <cds-input layout="vertical" control-width="shrink">
      <label>readonly input</label>
      <input placeholder="name" readonly />
    </cds-input>
  `;
}

/** @website */
export function inputWidth() {
  return html`
    <div cds-layout="vertical gap:lg">
      <cds-input layout="vertical" control-width="shrink">
        <label>vertical shrink width</label>
        <input placeholder="name" />
      </cds-input>

      <cds-input layout="horizontal" control-width="shrink">
        <label>horizontal shrink width</label>
        <input placeholder="name" />
      </cds-input>

      <cds-input layout="vertical" style="max-width: 300px">
        <label>vertical custom width</label>
        <input placeholder="name" />
      </cds-input>

      <cds-input layout="horizontal" style="max-width: 350px">
        <label>horizontal custom width</label>
        <input placeholder="name" />
      </cds-input>
    </div>
  `;
}

/** @website */
export function supportedTextTypes() {
  return html`
    <div cds-layout="vertical gap:lg">
      <cds-input>
        <label>Text</label>
        <input type="text" placeholder="name" />
      </cds-input>

      <cds-input>
        <label>Email</label>
        <input type="email" placeholder="test@test.com" />
      </cds-input>

      <cds-input>
        <label>Number</label>
        <input type="number" placeholder="age" />
      </cds-input>

      <cds-input>
        <label>Color</label>
        <input type="color" placeholder="color" value="#179BD3" />
      </cds-input>
    </div>
  `;
}

export function darkTheme() {
  return html`
    <cds-form-group layout="horizontal" cds-theme="dark">
      <cds-input layout="horizontal">
        <label>label</label>
        <input placeholder="name" />
        <cds-control-message>message text</cds-control-message>
      </cds-input>

      <cds-input layout="horizontal">
        <label>disabled</label>
        <input placeholder="name" disabled />
        <cds-control-message>disabled message</cds-control-message>
      </cds-input>

      <cds-input layout="horizontal" status="error">
        <label>error</label>
        <input placeholder="name" />
        <cds-control-message status="error">error message</cds-control-message>
      </cds-input>

      <cds-input layout="horizontal" status="success">
        <label>success</label>
        <input placeholder="name" />
        <cds-control-message status="success">success message</cds-control-message>
      </cds-input>
    </cds-form-group>
  `;
}

export function box() {
  return html`
    <style>
      [box] cds-input {
        --background: var(--cds-alias-object-container-background);
        --border-radius: var(--cds-alias-object-border-radius-100);
        --border: var(--cds-alias-object-border-width-100) solid var(--cds-alias-object-border-color);
        --border-bottom: 0;
        --padding: var(--cds-global-space-2) var(--cds-global-space-4) 0 var(--cds-global-space-4);
      }

      [box] cds-input input:focus {
        --border: var(--cds-alias-object-border-width-100) solid var(--cds-alias-status-info);
      }
    </style>
    <cds-form-group layout="vertical" box>
      <cds-input box>
        <label>box style</label>
        <input placeholder="name" />
        <cds-control-message>message text</cds-control-message>
      </cds-input>

      <cds-textarea box>
        <label>textarea</label>
        <textarea></textarea>
        <cds-control-message>message text</cds-control-message>
      </cds-textarea>
    </cds-form-group>
  `;
}
